# Generated by Django 4.1.1 on 2022-12-04 11:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('menu', '0002_menugroup_kind'),
    ]

    operations = [
        migrations.AddField(
            model_name='menu',
            name='is_new',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='menu',
            name='link',
            field=models.CharField(blank=True, default='#', max_length=255),
        ),
    ]



# from django.shortcuts import render

from django.views.generic import TemplateView

class IndexView(TemplateView):
    template_name = "index.html"

    def get_context_data(self, *args, **kwargs):
        context = super(IndexView, self).get_context_data(*args, **kwargs)

        # Menu Group -----------------------------------------
        #  Menu Group untuk membedakan menu site satu dan yg lain
        #  dan membedakan menu front end dan back end
        # Kind = 1 : FrontEND
        # kind = 2; Backend
        # menugroup = MenuGroup.objects.filter(site=self.site_id, kind=1)
        # if menugroup:
        #     context['menugroup'] = int(menugroup[0].id)
        #     # print('menugroup[0].id = ', menugroup[0].id)

        # else:
        #     raise Http404("Menu Group '%s' belum terdaftar, silahkan daftar di halaman <a href='%s'>admin</a>" % (request.get_host(), '/admin'))

        # Agency ---------------------------------------------
        # agency = get_agency_info(self.site_id)
        context['menugroup'] = '2'
        return context

class TestMenuView(TemplateView):
    template_name = 'test-menu.html'


from django.contrib import admin
from parler.admin import TranslatableAdmin

from .models import Menu, MenuGroup  # , MenuCustom

class MenuAdmin(admin.ModelAdmin):    
    list_filter = ('name', 'parent',) 
    list_display = ['name', 'parent', 'link', 'order_menu', 'kind', 'updated_at']
    search_fields = ('name', 'parent')
    ordering = ('-updated_at',)
# .order_by('parent_id','order_menu')

admin.site.register(Menu, TranslatableAdmin)

class MenuGroupAdmin(admin.ModelAdmin):
    list_filter = ('kind',) 
    list_display = ['group', 'site_domain', 'level', 'kind', 'updated_at']
    search_fields = ('group',)
    ordering = ('-level',)

admin.site.register(MenuGroup, MenuGroupAdmin)

# tidak di update model yg ini
# class MenuCustomAdmin(admin.ModelAdmin):
#     list_filter = ('menu',) 
#     list_display = ['menu', 'menu_group', 'updated_at']
#     search_fields = ('menu',)
#     ordering = ('-updated_at',)

# admin.site.register(MenuCustom, MenuCustomAdmin)


'''
    1. ----------------------------------------------------------------------------
    Parameter menu_group [Optional]
    Jika kosong, cek Site_ID (proses berdasarkan site ID menu default)
    Jika menu_group kosong, SITE_id harus ada, jika tidak maka ditidak datap di proses

    2. ----------------------------------------------------------------------------
    Di tabel tambahkan User_ID dan Site_ID
    Jika data SITE_ID saja yg ada maka ini untuk front end
    Jika User_ID saja yg ada maka ini sebagai backend

    3. ----------------------------------------------------------------------------
    Jika dua2nya ada, maka USER_ID saja yg di cek SITE_ID tidak di gunakan

    Update 4 Oktober 2022
'''

from django import template

# from ..models import Menu
from ..menus import Menus

register = template.Library()

# SET GLOBAL VARIABLE FORM MENU
 # cache nanti di template
 # variable global ini bukan untuk cache
global_menu = {} 
# format:
    # USER_ID, MENU_DATA, User_group, site_ID
    # Ada project_ID (untuk membedakan, project company_profile, acounting, etc)
# menugroup = MenuGroup.objects.filter(site=self.site_id, kind=1)
#         if menugroup:
#             context['menugroup'] = int(menugroup[0].id)
#             # print('menugroup[0].id = ', menugroup[0].id)

#         else:
#            raise Http404("Menu Group '%s' belum terdaftar, silahkan daftar di halaman <a href='%s'>admin</a>" % (request.get_host(), '/admin'))

# 2. Create Menu 
# class MenuCreate(template.Node):
#     '''
#         menu_kind :
#             FRONTEND = 1
#             BACKEND = 2 

#         menu_create FRONTEND menugroup as my_menu

#         menu_group :
#             jika = 0 maka front end, menu group kosong (ignore untuk mode frontend)
#     '''
#     def __init__(self, menu_kind, menu_group, var_name='menu'):       
#         self.menu_group = menu_group
#         self.menu_kind = menu_kind           
#         self.var_name = var_name
#         print(menu_group)
#         # print(request.get_host())

#     # 1. Cache server
#     #    Cek jika sudah ada data di variable maka tidak perlu ambil lagi di database
#     # def menu_cache():
#     #     lanjut = False
#     #     if global_menu:
#     #         if global_menu[self.menu_group]:

#     #     if not global_menu:
#     #         if self.menu_kind == 'FRONTEND':                  
#     #             my_menu = Menus(self.menu_group, 1)
#     #         else:                  
#     #             my_menu = Menus(self.menu_group, 2)
#     #         global_menu[self.menu_group] = my_menu
#     #         print('load from source')
#     #     else:
#     #         if not global_menu[self.menu_group]:
#     #             if self.menu_kind == 'FRONTEND':                  
#     #                 my_menu = Menus(self.menu_group, 1)
#     #             else:                  
#     #                 my_menu = Menus(self.menu_group, 2)
#     #             global_menu[self.menu_group] = my_menu
#     #             print('load from source')
#     #         else:
#     #             my_menu = global_menu[self.menu_group]
#     #             print('load from cache')   

#     def render(self, context):        
#         # print('group = ', self.menu_group)
#         # print('kinds = ', self.menu_kind)
#         # print('var_name = ', self.var_name)
        
#         # Parameter di template tidak perlu menggunakan tanda petik untuk menandakan string

#         # if not global_menu:
#         #     if self.menu_kind == 'FRONTEND':                  
#         #         my_menu = Menus(self.menu_group, 1)
#         #     else:                  
#         #         my_menu = Menus(self.menu_group, 2)
#         #     global_menu[self.menu_group] = my_menu
#         #     print('load from source')
#         # else:
#         #     if not global_menu[self.menu_group]:
#         #         if self.menu_kind == 'FRONTEND':                  
#         #             my_menu = Menus(self.menu_group, 1)
#         #         else:                  
#         #             my_menu = Menus(self.menu_group, 2)
#         #         global_menu[self.menu_group] = my_menu
#         #         print('load from source')
#         #     else:
#         #         my_menu = global_menu[self.menu_group]
#         #         print('load from cache') 

#         if self.menu_kind == 'FRONTEND':    
#             # print('oke')
#             print('self.menu_group = ', self.menu_group)
#             my_menu = Menus(self.menu_group, 1)
#         else:
#             my_menu = Menus(self.menu_group, 2)

#         global_menu['0'] = my_menu
#         context[self.var_name] = my_menu.get_menus()
#         return ''

# @register.tag(name='menu_create')            
# def get_menu_list(parser, token):
#     error = False
#     try :
#         tag_name, menu_kind, menu_group, _as, var_name = token.split_contents()
#         print('menu grou p= ', menu_group)
#         if _as != 'as':
#             error = True
#     except:
#         error = True

#     if error:
#         raise template.TemplateSyntaxError('menu_create must be of the form, "menu_create <menu_group> <menu_kind> as <var_name>"')
#     else:
#         return MenuCreate(menu_kind, menu_group, var_name)    


# Gunakan simple tag untuk create menu
# karena tidak bisa menerima variable di template tag
@register.simple_tag 
def menu_create(kind, menugroup):
    # if not global_menu: # var global menu masih kosong
    #     my_menu = Menus(menugroup, kind) # 1 = frontend
    #     global_menu['0'] = my_menu
    # else:
    #     my_menu = global_menu['0']

    # Pending update cache, belum disimpan site, dan tgl expired
    
    my_menu = Menus(menugroup, kind) # 1 = frontend
    global_menu['0'] = my_menu
    
    return my_menu.get_menus()
    # return ''



# ------------------------
# get active mnu

# @register.simple_tag(takes_context=True)
# def menu_active_simple_tag(context, active_menu, menu_group):    
#     my_menu = global_menu['1']
#     print('simple tag = ', my_menu.find_activeMenuList(active_menu))
#     return my_menu.get_menus()

# # OKE pakai simple tag
@register.simple_tag    #(takes_context=True) context, 
def menu_active(active_menu):    
    my_menu = global_menu['0']  # harus tipe data string    
    my_menu.get_active_menu(active_menu)
    return my_menu.get_list_active()

# @register.simple_tag # (takes_context=True) context
# def menu_breadcrumb(active_menu):    
#     # print('active_menu =' ,active_menu)
#     my_menu = global_menu['0']  # harus tipe data string  
#     # print(my_menu)  
#     return my_menu.create_breadcrumb(active_menu)


# OKE pakai simple tag untuk create bread crumb
# @register.simple_tag
# def menu_bread_crumb():
#     my_menu = global_menu['0']
#     print('bread  =', my_menu.create_breadCrumb('profile')    )
#     return my_menu.create_breadCrumb('profile')    


# UPDATE 22 Nop 2022
@register.simple_tag    #(takes_context=True) context, 
def menu_active_by_name(active_menu_name):    
    '''
        Jika active_menu_name mengandung underscore (_)
        maka ubah menjadi spasi
        karena di template jango tidak bisa menerima spasi untuk kondisi ini

        var active_menu = 'company name'
        {% menu_breadcrumb active_menu as my_active %}
        template akan error
        jadi solusinya ubah :
        var active_menu = 'company_name'
        proses di dalam fungsi ini ubah under score menjadi spasi
        kemudian baru bisa di compare dengan menu real.lowercase()
    '''

    # Update 22 Nop 22 :
    # tambahan .replace('_','')
    # replace underscore dengan spasi
    my_menu = global_menu['0']  # harus tipe data string    
    return my_menu.get_active_menu_by_name(active_menu_name.replace('_',' '))
    # return my_menu.get_list_active()


# Generated by Django 4.1.1 on 2023-01-26 14:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('menu', '0004_alter_menu_menu_group'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='menugroup',
            name='site',
        ),
        migrations.AddField(
            model_name='menugroup',
            name='group',
            field=models.OneToOneField(default=None, on_delete=django.db.models.deletion.CASCADE, to='auth.group'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='menugroup',
            name='level',
            field=models.SmallIntegerField(default=0),
        ),
    ]



'''
    Generate menus - create by ione
    Class ini untuk generate menu sesuai dengan data di database
    Date 13-02-21

    Update v2.0:
        # Sdh mengikuti update terakhir dari project OPD
        # Atur ulang tab
        # Ikuti model tabel menu yg ada di project ini atau OPD

    Update v2.1:
        # create menu berdasarkan user yg login
        # untuk front end, karena tidak login maka generate menu berdasarkan
        # user yg berada di site tersebut

    Update v2.2:
        # perbaikan bread crumb menu
        # tambah bread crumb lengkap dengan link (tidak hanya name saja)

    Update v3.0:
        # using recursive function, all result list same as old version
'''

# Tambah dictionary sesuai data di database, Algoritma :
# 1. Buat query Order by parent id dan order menu
# 2. copy data ke mDict record per pecord
# 3. Append ke mList, sehingga jadinya list of dictionary
# 4. Atur ulang mList, sehingga urutannya menjadi :
#    Menu Parent
#       |--- Menu Anak1
#       |--- Menu Anak2, dst...
# 5. Atur Ulang Level, sehingga menjadi :
#    Dictionary memiliki item baru
#       1. Level : untuk menentukan kedalaman level menu anak
#       2. HaveChild : True or False, Untuk penanda apakah menu parent mempunyai anak atau tidak
#       3. haveChildEndTag : Type Integer, Untuk penanda tag penutup menu anak di buat
#          Jika level 1, buat satu tag penutup menu anak
#          Jika level 2, buat dua tag penutup menu anak, dst...
# 6. Atur active menu
# Buat function baru untuk Atur BreadCrumb, tambahkan List of Dictionary dengan format 
#       [{'Nama','href'},{'nama2','href2'}]
#       Data ini akan di cek di template dengan cara yang sama seperti generate menu
#  --------------------------------------------------------------------------------------------------          

from django.db.models import F
from .models import Menu # MenuCustom


class Menus:    
    # mLvl_prev = -1   # catat level sebelumnya  
    mList_recursive = []
    mList_active = []
    menu_custom_list = []   # exclude this menu from default menus
    # mDict = {}     
    # mList = []      # result ada di mList
    site_id = 1
    lang = 'id'
    group_id = 0      # simpan group_id untuk digunakan di recursive

    # defailt menu_group = 0 artinya all
    # kinds = 1 Front end
    # kinds =2 backend
    # kinds =0 all
    def __init__(self, menu_group = 0, kinds = 0, site_id = 1): #, pIs_master_menu = False):         # menu_group adalah filter untuk company tertentu saja
        '''
            Jika pKind = 0 maka ambil data semua, frontend dan backend
        '''
        self.site_id = site_id

        # get active language
        obj = Menu()
        self.lang = obj.get_current_language()
        self.group_id = menu_group

        if len(self.mList_recursive) == 0:
            #if menu_group != "":
            self.create_menus(menu_group, kinds) #, pIs_master_menu)
        else:
            # self.mDict = {}  # clear dulu (karena prosedur init ini sekali dijalankan saat class di buat)
            self.mList_recursive = []
            self.mList_active = []
            self.create_menus(menu_group, kinds) #, pIs_master_menu)                    

    def get_menus(self):
        return self.mList_recursive

    def get_list_active(self):
        return self.mList_active

    # def get_menu_custom_list(self, menu_group):
    #     # select custom menu for ignoring
    #     self.menu_custom_list = []
    #     menu_custom = list(MenuCustom.objects.filter(site_id=self.site_id). \
    #         exclude(menu_group_id=menu_group).values('menu_id'))
    #     # print(menu_custom)

    #     for i in menu_custom:
    #         self.menu_custom_list.append(i['menu_id'])

    #     # print(self.menu_custom_list)

    def ignore_circular_parent(self):
        # 0. Sebelum proses menu, update dulu seluruh menu yg id = id parent set id parent = NULL untuk menghindari
        Menu.objects.filter(id=F('parent_id')).update(parent_id=None)    # query pengaman

    def create_menus(self, menu_group, kinds):     
        # 0. Sebelum proses menu, update dulu seluruh menu yg id = id parent set id parent = NULL untuk menghindari
        # Menu.objects.filter(id=F('parent_id')).update(parent_id=None)    # query pengaman
        # 1. Clear circular reference (ignore it, or set as None)
        self.ignore_circular_parent()
       
        # exclude custom menu tidak digunakan disini karena hanya di ambil root menu saja
        # [UPDATE] tetap digunakan untuk antisipasi root menu digunakan sebagai custom menu
        # self.get_menu_custom_list(menu_group)

        # get active language
        
        # 2. Get data by user options (UPDATE only get ROOT MENU base on User OPTION)
        # Harus konversi ke integer karena tidak masuk ke kondisi        
        if int(kinds) == 0: # jika kind = 0 ambil semua data front end dan back end            
            # .exclude(id__in=self.menu_custom_list) \
            # UPADATE menu_group__id menjadi menu_group__group_id
            mData = Menu.objects.language(self.lang).filter(menu_group__group_id=menu_group \
                ,is_visibled=True, parent=None) \
                .order_by('parent_id','order_menu').values('id')     
        # elif int(menu_group) == 0:   # menu group = 0 artinya menu frontend            
        #     # .exclude(id__in=self.menu_custom_list) \
        #     mData = Menu.objects.language(self.lang).filter(kind=kinds, is_visibled=True, parent=None) \
        #         .order_by('parent_id','order_menu').values('id')     

        # !!!!!!! UPDATE !!!!!!!
        # ----------------------
        # Update 30 januari 2023, Frontend juga harus jelas menugroup milik frontend (Annonymous)
        # kondisi ambil semua yg bertipe front end tidak ada lagi
        # karena front end juga bisa berbeda susunan menunya tergantung user yg lagi aktif (default annonymous)

        # kondisi is_visible dan menu_group_id di perlukan untuk cek apakah menu root visible dan benar milik user yg aktif
        # kondisi ini juga perlu di cek di recursive create menu, untuk mendeteksi sub menu is_visible dan milik user aktif atau bukan
        # ---------------------

        else:   
            # .exclude(id__in=self.menu_custom_list) \         
            mData = Menu.objects.language(self.lang).filter(menu_group__group_id=menu_group \
                ,kind=kinds, is_visibled=True, parent=None) \
                .order_by('parent_id','order_menu').values('id')                  

        # .exclude(id__in=menu_custom_list) \
        # 3. Get root menu
        # get menu id only
        # print(mData)
        # menu_id = []
        # for i in mData:
        #     menu_id.append(i.id)

        # Entry point from root menu
        # root_menu = self.get_root_menu(menu_id)
        root_menu = []
        for i in mData:
            root_menu.append(i['id'])

        print('root_menu=', root_menu)

        # print('root_menu', root_menu)

        # 4. begin process recursive menu
        self.create_menu_recursive(root_menu,0)

        # 5. Update End Tag
        self.update_end_tag()

        # 6. get complete data base on mData
        self.get_menus_complete()

        # 6. return result
        #return self.get_menus()

    def get_menus_complete(self):
        # obj = Menu()
        # lang = obj.get_current_language()

        mCount = 0
        while mCount < len(self.mList_recursive):
            mData = Menu.objects.language(self.lang).get(id=self.mList_recursive[mCount]['id'])
            # print(mData.order_menu)
            self.mList_recursive[mCount]['uuid'] = mData.uuid
            self.mList_recursive[mCount]['order_menu'] = mData.order_menu
            self.mList_recursive[mCount]['name'] = mData.name
            self.mList_recursive[mCount]['link'] = mData.link
            self.mList_recursive[mCount]['icon'] = mData.icon
            self.mList_recursive[mCount]['is_external'] = mData.is_external
            self.mList_recursive[mCount]['is_visibled'] = mData.is_visibled
            self.mList_recursive[mCount]['is_new'] = mData.is_new
            self.mList_recursive[mCount]['parent_id'] = mData.parent_id
            # self.mList_recursive[mCount]['order_menu'] = mData.order_menu
            

            mCount += 1

    # def get_root_menu(self, menu_id):
    #     data = Menu.objects.filter(id__in=menu_id, parent=None).values('id')        
    #     ret = []
    #     for i in data:
    #         ret.append(i['id'])

    #     return ret

    def is_have_child(self, menu_id):
        '''
            cek apakah ada menu dengan parent = menu_id?
            jika ya return True, else False
        '''
        # tidak perlu lang disini karena tidak ada field name di ambil
        # .exclude(id__in=self.menu_custom_list) \
        data = Menu.objects.filter(parent_id=menu_id \
            ,menu_group__group_id=self.group_id \
            ,is_visibled=True) \
            .order_by('parent_id','order_menu').values('id')        
        ret = []
        for i in data:
            ret.append(i['id'])

        return ret

    def create_menu_recursive(self, root_menu_id, lvl): # , menu_group, kinds):     
        '''
            menu_group, kinds : untuk mendapatkan kondisi where, data selalu sama
            root_menu_id : berisi ID menu root untuk level 0
                           berisi ID menu[0] (data satu index saja) untuk level 1..n
                           Hanya berisi ID dengan format ['id_1', 'id_2', 'id_n']
        '''        
        # print(root_menu_id)

        for i in root_menu_id:
            child_id = self.is_have_child(i)
            if child_id:
                self.mList_recursive.append({'id':i, 'level':lvl, 'haveChild':True})
                lvl += 1                                    
                self.create_menu_recursive(child_id, lvl)                
                lvl -= 1
            else:
                self.mList_recursive.append({'id':i, 'level':lvl, 'haveChild':False})

    def update_end_tag(self):
        '''
            parameter mList_recursive
            cari yang next level turun (misal dari 1 ke 0, atau dari 2 ke 1, 2 ke 0, semua yg menurun tandai sebagai true end_tag nya)
        '''
        mCount = 0
        while mCount < len(self.mList_recursive):
            # kondisi akhir 
            if mCount == len(self.mList_recursive)-1:
                if self.mList_recursive[mCount]['level'] == 0:  # kondisi tag akhir 0, langsung set end tag = 1
                    self.mList_recursive[mCount]['haveChildEndTag'] = [1]
                else:
                    tmp = self.mList_recursive[mCount]['level']
                    self.mList_recursive[mCount]['haveChildEndTag'] = list(range(1,tmp+1))

            # kondisi lvl skr dan next sama2 nol (root)
            elif self.mList_recursive[mCount]['level'] == 0 and self.mList_recursive[mCount+1]['level'] == 0:
                self.mList_recursive[mCount]['haveChildEndTag'] = [1]

            # kondisi lvl menurun
            else:
                selisih = self.mList_recursive[mCount]['level'] - self.mList_recursive[mCount+1]['level']
                if selisih > 0:
                    self.mList_recursive[mCount]['haveChildEndTag'] = list(range(1,selisih+1))
                else:
                    self.mList_recursive[mCount]['haveChildEndTag'] = []

            mCount += 1

    # Menu bisa diberikan dengan nama yg sama, oleh karena itu cari active menu dan breadcrumb menggunakan ID
    def get_active_menu(self, menu_id):
        '''
            Find active menu recursively
        '''        
        data = Menu.objects.language(self.lang).filter(id=menu_id)
                # .exclude(id__in=self.menu_custom_list) 
        if data:
            for i in data:
                # print(i.parent_id)
                parent_id = i.parent_id
                self.mList_active.insert(0, {'id': i.id, 'name': i.name, 'link': i.link, 'icon': i.icon, 'is_external':i.is_external})
                self.get_active_menu(parent_id)
        
    # function ini sama seperti diatas, hanya saja parameter berupa menu name
    # find dulu id dari menu name, jika di temukan lebih dari satu munculkan warning
    # data yg sesuai lebih dari satu
    def get_active_menu_by_name(self, menu_name):
        #data = Menu.objects.filter()
        # cari data yg sesuai di self.mList_recursive
        # tidak perlu baca database lagi
        self.mList_active.clear()
        # print('menu_name = ', menu_name)

        for i in self.mList_recursive:
            # print(i['name'].lower(), menu_name)

            if i['name'].lower() == menu_name.lower():
                parent_id = i['parent_id']
                self.mList_active.insert(0, {'id': i['id'], 'name': i['name'], 'link': i['link'], 'icon': i['icon'], 'is_external':i['is_external']})

                # recursive ke get_active_menu BY ID
                self.get_active_menu(parent_id)

        result_list = []
        for i in self.mList_active:
            result_list.append(i['name'])

        return result_list
        


# Generated by Django 4.1.1 on 2023-01-26 15:14

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('auth', '0012_alter_user_first_name_max_length'),
        ('menu', '0005_remove_menugroup_site_menugroup_group_and_more'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='menugrouptranslation',
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name='menugrouptranslation',
            name='master',
        ),
        migrations.AddField(
            model_name='menugroup',
            name='site',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site'),
        ),
        migrations.AlterField(
            model_name='menugroup',
            name='group',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.group'),
        ),
        migrations.DeleteModel(
            name='MenuCustom',
        ),
        migrations.DeleteModel(
            name='MenuGroupTranslation',
        ),
    ]



# Generated by Django 4.1.1 on 2023-01-30 11:41

from django.db import migrations
from menu.auto_insert_group import create_groups

class Migration(migrations.Migration):

    dependencies = [
        ('menu', '0006_alter_menugrouptranslation_unique_together_and_more'),
    ]

    operations = [
        migrations.RunPython(create_groups)
    ]



'''
    Recreate model base on OPD project and Outbox Project
    29th September 2022
    Grid Software, Inc.
'''
import uuid

from django.contrib.auth.models import Group
from django.contrib.sites.models import Site
from django.db import models
from django.db.models import Max
# from django.contrib.auth.models import User
# from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from parler.models import TranslatableModel, TranslatedFields

# User = get_user_model()

class OptMenuKinds(models.IntegerChoices):
    '''
        Jenis menu : 
            0 : Front end without user login
            1 : Front end with user login
            2 : Back end with user login (nothing for none user)

            Update 17 Sept 2021
            0 : Front End
            1 : Back End
    '''
    FRONTEND = 1, _('Frontend')
    BACKEND = 2, _('Backend') 

    # Tidak jadi karena crash dengan unique together di model
    # BACKEND_DEFAULT = 3 # Penanda user baru login, belum ada company yg aktif, sehingga menu belum bisa di generate
                        # untuk itu generate menu default langsung   

class MenuGroup(models.Model):
    '''
        Group : Model Menu
        Simpan data group menu :
        1. Owner
        2. Manager
        3. Operator
        4. Kasir
        5. dll...

        Tidak ada interface, maintenance di halaman admin


        # Perubahan proses bisnis :
        --------------------------

        1. MenuGrup ini berelasi OneToOne ke tabel auth group django
        agar supaya permission mengikuti setting default django

        2. Site tetap dipertahankan, karena setiap site memiliki struktur group yang sama
        tidak ada akses untuk client mengubah struktur group ini
        Jika site kosong itu artinya group menu milik semua site

        2.a name dihapus karena sudah ada di auth group django
        2.b menu customs di hapus karena menu custom dapat langsung di create di menu utama

        3. Struktur group default adalah :
            # admin (superuser), akses melalui secret-admin (hanya dimiliki oleh owner code)
                security level 10
                # akun developer, akses melalui dashboard (ada menu khusus yg hanya muncul di group ini)
                    security level 9
                    # akun client (admin-client), akses melalui dashboard, memiliki akses untuk untuk membuat sub user di bawah client
                        security level 8
                        # akun operator-client, memiliki akses untuk membuka menu tertentu saja
                        security level 7
                        client dapat berkreasi membuat nama operator-client ini
                        nama default yg telah ada di database :
                            # owner     level 6
                            # manager   level 5
                            # operator  level 4

        4. Urutan group backend (kind=2) :
            9. Super Admin
            8. Developer
            7. Admin
            6. Owner
            5. Manager
            4. Operator
            3. ...
            2. ...
            1. ...

           Urutan group frontend (kind=1):
            3. ...
            2. ...
            1. ...
            0. Anonymous     

            level 1..3 untuk group tertentu misal user telah login, ada menu khusus yg muncul untuk group tsb.

        5. id yang digunakan adalah id dari auth_group bukan id dari menuGroup
    '''
    # Update 19 Oktober 2022
    # untuk membedakan menu project 1 dengan yg lain, dengan nama sama
    site = models.ForeignKey(Site, on_delete=models.CASCADE, null=True, blank=True)
    # site ada tambahan null true, blank true
    # jika berisi null, itu artinya milik semua site

    uuid = models.UUIDField(unique=True, default=uuid.uuid4, editable=False)

    # additional fields
    # null true, untuk compatibilitas dengan versi sebelumnya
    group = models.OneToOneField(Group, on_delete=models.CASCADE, null=True, blank=True)
    level = models.SmallIntegerField(default=0)     

    # Tidak boleh ada data kembar di name    
    # translations = TranslatedFields(
    #     name = models.CharField(max_length=100) # unique=True (Unique together with site)
    # )
    
    # Untuk membedakan mana menu group front end mana yg backend
    kind = models.SmallIntegerField(choices=OptMenuKinds.choices, default=OptMenuKinds.FRONTEND)

    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)    

    # Optional Fields:
    # created_at = models.DateTimeField(auto_now_add=True)
    # updated_at = models.DateTimeField(auto_now=True)

    # objects = MenuGroupManager()
    
    # def __str__(self):
    # def __unicode__(self):
    def __str__(self):
        return self.group.name

    def site_domain(self):
        return self.site.domain if self.site else 'All'

    # class Meta:
    #     ordering = ('-level',)

    # class Meta:
    #     constraints = [
    #         models.UniqueConstraint(fields=['site', 'translations__name'], name='unique_site_name')
    #     ]

# def company_name_validate(value):
#     if len(value) < 3:
#         raise ValidationError("Company name mush be more than 3 character")
#     else:
#         return value

class Menu(TranslatableModel):
    '''    
        Relase :
            User --1:N-- Menu
        Deskripsi :
            # Menu di buat per user,
            # Satu company bisa mempunyai banyak user,
            # Satu User bisa punya banyak menu
            # Tetapkan satu user sebagai user master (berguna sebagai menu master), di copy ke menu user lain
    
        # ada 1 company dengan site ID = 1 sebagai master menu
        # seluruh menu di create dari site ini (berarti is_master_menu tidak di perlukan lagi)
        # Untuk menu di app blog, user tidak dalam kondisi login
        # Untuk menu di app inventory, user harus login dulu        
        # Jika user telah di create, langsung generate menu untuk user tersebut
        # dapat di visible sesuai kebutuhan
    '''      
    # ID tetap di create otomaris
    uuid = models.UUIDField(unique=True, default=uuid.uuid4, editable=False) 
    translations = TranslatedFields(
        name = models.CharField(max_length=100)
    )

    # menu group, relasi many to many 
    # Untuk frontend menu group kosong
    menu_group = models.ManyToManyField(MenuGroup, related_name="menu_group_menu", blank=True) # , null=True, blank=True)  not effect to m2m relation

    # Optional Fields:
    # user = models.ForeignKey(User, null=True, blank=True, on_delete=models.PROTECT)        # Menu di generate per user   
    # user harus diimport dulu

    parent = models.ForeignKey('self', null=True, blank=True, on_delete=models.CASCADE)    

    # link: adalah name menu yg di slugify
    # href di ubah menjadi link
    # jika ada ://www maka dianggap link luar
    # /home/dashboard is not valid URLField (change back to CharField)
    # link = models.URLField(max_length=255, null=True, blank=True)  # , verbose_name='Link'

    # Update 4 Desember 22
    link = models.CharField(max_length=255, default='#', blank=True)  # Tambah # agar tidan None di link menu # null=True, 

    # urut menu (ambil dari id, tapi user dapat mengubah sendiri ordernya)
    order_menu = models.SmallIntegerField(default=0)

    # awesome icon
    # data berupa blok <i class="icon"></i>
    icon = models.CharField(max_length=50, null=True, blank=True)    

    #is_admin_menu = models.BooleanField(default=False)	    
    # is_admin_menu: jika true, menu untuk backend, jika false menu untuk frontend
    # jenis menu
    kind = models.SmallIntegerField(choices=OptMenuKinds.choices, default=OptMenuKinds.FRONTEND)

    # is_visibled: untuk menyembunyikan menu
    is_visibled = models.BooleanField(default=True)	    

    # is_master_menu: adalah menu yg digunakan oleh banyak user
    # tidak bisa dihapus, hanya super user yg bisa menghapus
    # is_master_menu adalah menu yg di copy oleh user lain saat pertama kali di buat

    # Update 17 Sept 2021 
    # is_master_menu tidak digunakan lagi karena seluruh menu di create sesuai dengan
    # data menu di SITE 1
    # (menu di SITE 1 sebagai menu master yg akan di copy ke site lain saat User pertama kali di create)
    # is_master_menu = models.BooleanField(default=False)	 

    # is_statis_menu: penanda halaman statis
    # semua menu yg di create oleh user adalah statis menu
    # diakses khusus melalui link tertentu di halaman statis   
    # is_statis_menu = models.BooleanField(default=False)	
    is_external = models.BooleanField(default=False)	    # Jika external menu True, maka otomatis target _BLANK

    # Penanda menu baru untuk di dashboard (Input dari halaman secret-admin)
    # Update 3 Desember 2022
    is_new = models.BooleanField(default=False)	    

    # timestamp
    # created_at = models.DateTimeField(auto_now_add=True)
    # updated_at = models.DateTimeField(auto_now=True)

    # untuk status sinkronisasi ke server
    # penanda di server lokal apakah sudah di sinkronisasi atau belum
    # yg paling penting tanggal harus valid 
    # is_already_sync = models.BooleanField(default=False)	

    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)    

    # def __str__(self):
    #     return self.name

    # class Meta:
    #     constraints = [
    #         models.UniqueConstraint(fields=['menu_group', 'name'], name='unique_menu_group_name')
    #     ]

    # def updated_at_(self):
    #     return get_natural_datetime(self.updated_at)

    # def created_at_(self):
    #     return get_natural_datetime(self.created_at)

    # def parent_(self):
    #     if self.parent:
    #         return self.parent.name
    #     else:
    #         return '-'

    # def updated_at_(self):
    #     return get_natural_datetime(self.updated_at)        
    
    # def updated_at_(self):
    #     return  "naturalday(self.updated_at)"
       # return serializers.serialize("json",' naturalday(self.updated_at)')
        #return serializers.serialize('json', naturalday(self.updated_at))

    # class meta:
    #     ordering = ('-parent_id', '-order_menu',)

    # def __str__(self):          
    def __str__(self):
        if self.kind == OptMenuKinds.FRONTEND:
            res = '[ Front-End ]'       # halaman depan
        # elif self.kind == OptMenuKinds.BACKEND_DEFAULT:
        #     res = '[ Default ]'
        else:
            res = '[ Back-End ]'        # halaman dashboard

        # res = get_kind_display() # tidak ada selfnya kemungkinan tidak bisa

        if self.parent:
            par = self.parent.name      # tampilkan name parent jika ada    
        else:
            par = 'ROOT'

        return "{} {} > {}".format(res, par, self.name)      

    # untuk slug
    def save(self, *args, **kwargs):   
        # menggunakan ID bisa menyebabkan data overload karena tipe data ID BIGINT
        if self.order_menu == 0:     
            #     self.order_menu
            # print('menu_group = ' , self.menu_group.exists())
            # print('parent = ' , self.parent_id)
            # print('kind = ' , self.kind)
            obj = Menu.objects.filter(parent=self.parent, kind=self.kind).aggregate(max=Max('order_menu'))
            if obj:
                # max = obj['max'] + 1
                if obj['max']:
                    self.order_menu = obj['max'] + 1

        # print(tmp)

        super(Menu, self).save(*args, **kwargs)   


# class MenuCustom(models.Model):
#     '''
#         # Akan di hapus di versi berikutnya
#         # status deprecated

#         # tidak ada update ke multi language khusus model ini
#         Custom menu adalah menu yg hanya muncul di site dan menu_group tertentu saja
#         tidak muncul di tempat lain
#     '''
#     # Menu ini digunakan untuk filter custom menu yg ada di site, exlude menu_group ada di get_menu_custom_list menus.py
#     site = models.ForeignKey(Site, on_delete=models.CASCADE)

#     menu_group = models.ForeignKey(MenuGroup, on_delete=models.PROTECT) #, blank=True, null=True)    
#     menu = models.OneToOneField(Menu, on_delete=models.CASCADE) # One to One Relations to menu

#     created_at = models.DateTimeField(auto_now_add=True, editable=False)
#     updated_at = models.DateTimeField(auto_now=True, editable=False)    


#     def __str__(self):  
#         return "{}".format(self.menu)  




# Generated by Django 4.1.1 on 2022-11-25 09:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('menu', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='menugroup',
            name='kind',
            field=models.SmallIntegerField(choices=[(1, 'Frontend'), (2, 'Backend')], default=1),
        ),
    ]



"""
Provides menu version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update menu` to change this file.

from incremental import Version

__version__ = Version("menu", 1, 0, 19)
__all__ = ["__version__"]



from .models import MenuGroup
from django.contrib.auth.models import Group

def create_groups(apps, schema_monitor):
    # auth_group = apps.get_model('django.contrib.auth', 'Group')

    # For backend
    mlist = [   
                'Super Admin',  # lvl 9
                'Developer',    # lvl 8
                'Admin',        # lvl 7
                'Owner',        # lvl 6
                'Manager',      # lvl 5
                'Operator'      # lvl 4
            ]

    mlevel = 9
    for i in mlist:                
        group, created = Group.objects.get_or_create(name=i)
        if created:
            MenuGroup.objects.create(group=group, kind=2, level=mlevel)
        mlevel -= 1

    # for frontend
    mlevel = 0
    mlist = [   
                'Anonymous',  # lvl 0
            ]
    for i in mlist:                
        group, created = Group.objects.get_or_create(name=i)
        if created:
            MenuGroup.objects.create(group=group, kind=1, level=mlevel)
        mlevel -= 1

    # end (run on migrate)

